// -*- mode: C++ -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_Metrics_h
#define mozilla_Metrics_h

#include "mozilla/glean/bindings/MetricTypes.h"
#include "mozilla/Tuple.h"
#include "mozilla/Maybe.h"
#include "nsTArray.h"
#include "nsPrintfCString.h"

namespace mozilla::glean {
struct NoExtraKeys;

namespace browser_launched_to_handle {
    /**
     * generated from browser.launched_to_handle.system_notification
     */
    struct SystemNotificationExtra {
      mozilla::Maybe<nsCString> action;
      mozilla::Maybe<nsCString> name;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (action) {
          extraKeys.AppendElement()->AssignASCII("action");
          extraValues.EmplaceBack(action.value());
        }
        if (name) {
          extraKeys.AppendElement()->AssignASCII("name");
          extraValues.EmplaceBack(name.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when Firefox launches to complete a native notification popped by a
     * system (chrome privileged) alert.  Windows-only at the time of writing.
     */
    constexpr impl::EventMetric<SystemNotificationExtra> system_notification(1);

}
namespace newtab {
    /**
     * generated from newtab.locale
     */
    /**
     * The application's locale as of when newtab's TelemetryFeed was init. Comes from
     * `Services.local.appLocaleAsBCP47`. Looks like `en-US`.
     */
    constexpr impl::StringMetric locale(2);

    /**
     * generated from newtab.newtab_category
     */
    /**
     * The current setting of the newtab page. One of ["enabled", "disabled",
     * "extension"] or any value from SiteClassifier like "known-hijacker" or "social-
     * media". Similar to Activity Stream's PAGE_TAKEOVER_DATA event's
     * `newtab_url_category`. Sampled once after newtab init.
     */
    constexpr impl::StringMetric newtab_category(3);

    /**
     * generated from newtab.homepage_category
     */
    /**
     * The current setting of the home page. One of ["enabled", "disabled",
     * "extension"] or any value from SiteClassifier like "known-hijacker" or "social-
     * media". Similar to Activity Stream's PAGE_TAKEOVER_DATA event's
     * `home_url_category`. Sampled once after newtab init.
     */
    constexpr impl::StringMetric homepage_category(4);

    /**
     * generated from newtab.opened
     */
    struct OpenedExtra {
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> source;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (source) {
          extraKeys.AppendElement()->AssignASCII("source");
          extraValues.EmplaceBack(source.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when newtab UI is opened via `about:newtab` or `about:home` or
     * `about:welcome` and has been made visible (see `visibility_event_rcvd_ts` in
     * [detect-user-session-start.js](https://searchfox.org/mozilla-
     * central/source/browser/components/newtab/content-src/lib/detect-user-session-
     * start.js)).
     */
    constexpr impl::EventMetric<OpenedExtra> opened(5);

    /**
     * generated from newtab.closed
     */
    struct ClosedExtra {
      mozilla::Maybe<nsCString> newtabVisitId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when newtab UI is closed by * navigation * closing the tab
     * Doesn't mean that the newtab was ever visible to a user.
     */
    constexpr impl::EventMetric<ClosedExtra> closed(6);

}
namespace newtab_search {
    /**
     * generated from newtab.search.enabled
     */
    /**
     * Whether the search input is enabled on the newtab. Corresponds to the value of
     * the `browser.newtabpage.activity-stream.showSearch` pref.
     */
    constexpr impl::BooleanMetric enabled(7);

    /**
     * generated from newtab.search.issued
     */
    struct IssuedExtra {
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> searchAccessPoint;
      mozilla::Maybe<nsCString> telemetryId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (searchAccessPoint) {
          extraKeys.AppendElement()->AssignASCII("search_access_point");
          extraValues.EmplaceBack(searchAccessPoint.value());
        }
        if (telemetryId) {
          extraKeys.AppendElement()->AssignASCII("telemetry_id");
          extraValues.EmplaceBack(telemetryId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * When Firefox was asked to issue a search from a Search Access Point (SAP) on a
     * newtab page. Doesn't record searches in Private Browsing Mode unless
     * `browser.engagement.search_counts.pbm` is set to `true`.
     */
    constexpr impl::EventMetric<IssuedExtra> issued(8);

}
namespace topsites {
    /**
     * generated from topsites.enabled
     */
    /**
     * Whether "topsites" is enabled on the newtab. AKA the "Shortcuts" section.
     * Corresponds to the value of the `browser.newtabpage.activity-
     * stream.feeds.topsites` pref.
     */
    constexpr impl::BooleanMetric enabled(9);

    /**
     * generated from topsites.sponsored_enabled
     */
    /**
     * Whether sponsored topsites are enabled on the newtab. AKA the "Sponsored
     * Shortcuts" section. Corresponds to the value of the
     * `browser.newtabpage.activity-stream.showSponsoredTopSites` pref. Can be `true`
     * even if topsites.enabled is `false`.
     */
    constexpr impl::BooleanMetric sponsored_enabled(10);

    /**
     * generated from topsites.impression
     */
    struct ImpressionExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when topsite tiles are loaded. Presently only happens for sponsored
     * tiles.
     */
    constexpr impl::EventMetric<ImpressionExtra> impression(11);

    /**
     * generated from topsites.click
     */
    struct ClickExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a topsite tile is clicked. Presently only happens for sponsored
     * tiles. Only happens on click. Not on middle-click. Not on "Open in new
     * Tab"-like options in the context menu.
     */
    constexpr impl::EventMetric<ClickExtra> click(12);

}
namespace pocket {
    /**
     * generated from pocket.is_signed_in
     */
    /**
     * Whether the Firefox user is signed in to Pocket. Does not correspond to a pref,
     * so its value is resampled at newtab's component init and whenever there is a
     * Discovery Stream user event.
     */
    constexpr impl::BooleanMetric is_signed_in(13);

    /**
     * generated from pocket.enabled
     */
    /**
     * Whether Pocket is enabled on the newtab. AKA the "Recommended by Pocket"
     * section. Corresponds to the value of the `browser.newtabpage.activity-
     * stream.feeds.section.topstories` pref.
     */
    constexpr impl::BooleanMetric enabled(14);

    /**
     * generated from pocket.sponsored_stories_enabled
     */
    /**
     * Whether Pocket sponsored stories are enabled on the newtab. Corresponds to the
     * value of the `browser.newtabpage.activity-stream.showSponsored` pref. Can be
     * `true` even if pocket.enabled is `false`.
     */
    constexpr impl::BooleanMetric sponsored_stories_enabled(15);

    /**
     * generated from pocket.impression
     */
    struct ImpressionExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<uint32_t> position;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (position) {
          extraKeys.AppendElement()->AssignASCII("position");
          extraValues.EmplaceBack(nsPrintfCString("%d", position.value()));
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a pocket tile is visible to the user.
     */
    constexpr impl::EventMetric<ImpressionExtra> impression(16);

    /**
     * generated from pocket.click
     */
    struct ClickExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<uint32_t> position;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (position) {
          extraKeys.AppendElement()->AssignASCII("position");
          extraValues.EmplaceBack(nsPrintfCString("%d", position.value()));
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a pocket tile is clicked. Only happens on click. Not on middle-
     * click. Not on "Open in new Tab"-like options in the context menu.
     */
    constexpr impl::EventMetric<ClickExtra> click(17);

    /**
     * generated from pocket.save
     */
    struct SaveExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<uint32_t> position;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (position) {
          extraKeys.AppendElement()->AssignASCII("position");
          extraValues.EmplaceBack(nsPrintfCString("%d", position.value()));
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a user decides to save a pocket tile. Does not mean it ends up
     * successfully saved. Just that the user clicked on "Save to Pocket" in the
     * little pocket tile menu.
     */
    constexpr impl::EventMetric<SaveExtra> save(18);

    /**
     * generated from pocket.topic_click
     */
    struct TopicClickExtra {
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> topic;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (topic) {
          extraKeys.AppendElement()->AssignASCII("topic");
          extraValues.EmplaceBack(topic.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a pocket "Popular Topic" is clicked. Only happens on click. Not
     * on middle-click. Not on "Open in new Tab"-like options in the context menu.
     */
    constexpr impl::EventMetric<TopicClickExtra> topic_click(19);

}
namespace newtab_search_ad {
    /**
     * generated from newtab.search.ad.impression
     */
    struct ImpressionExtra {
      mozilla::Maybe<bool> isFollowOn;
      mozilla::Maybe<bool> isTagged;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> searchAccessPoint;
      mozilla::Maybe<nsCString> telemetryId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isFollowOn) {
          extraKeys.AppendElement()->AssignASCII("is_follow_on");
          extraValues.AppendElement()->AssignASCII(isFollowOn.value() ? "true" : "false");
        }
        if (isTagged) {
          extraKeys.AppendElement()->AssignASCII("is_tagged");
          extraValues.AppendElement()->AssignASCII(isTagged.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (searchAccessPoint) {
          extraKeys.AppendElement()->AssignASCII("search_access_point");
          extraValues.EmplaceBack(searchAccessPoint.value());
        }
        if (telemetryId) {
          extraKeys.AppendElement()->AssignASCII("telemetry_id");
          extraValues.EmplaceBack(telemetryId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a newtab visit resulted in a search that loaded a Search Engine
     * Result Page (SERP) that contains an ad link. And the SERP is visible.
     */
    constexpr impl::EventMetric<ImpressionExtra> impression(20);

    /**
     * generated from newtab.search.ad.click
     */
    struct ClickExtra {
      mozilla::Maybe<bool> isFollowOn;
      mozilla::Maybe<bool> isTagged;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> searchAccessPoint;
      mozilla::Maybe<nsCString> telemetryId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isFollowOn) {
          extraKeys.AppendElement()->AssignASCII("is_follow_on");
          extraValues.AppendElement()->AssignASCII(isFollowOn.value() ? "true" : "false");
        }
        if (isTagged) {
          extraKeys.AppendElement()->AssignASCII("is_tagged");
          extraValues.AppendElement()->AssignASCII(isTagged.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (searchAccessPoint) {
          extraKeys.AppendElement()->AssignASCII("search_access_point");
          extraValues.EmplaceBack(searchAccessPoint.value());
        }
        if (telemetryId) {
          extraKeys.AppendElement()->AssignASCII("telemetry_id");
          extraValues.EmplaceBack(telemetryId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when an ad link is clicked on a Search Engine Result Page (SERP) which
     * was loaded by a seach that began on a newtab page.
     */
    constexpr impl::EventMetric<ClickExtra> click(21);

}
namespace browser_engagement {
    /**
     * generated from browser.engagement.active_ticks
     */
    /**
     * The number of five-second intervals ('ticks') the user was considered
     * 'active'.
     * 
     * 'active' means keyboard or mouse interaction with the application.
     * It doesn't take into account whether or not the window has focus or is in
     * the foreground, only if it is receiving these interaction events.
     * 
     * Migrated from Telemetry's `browser.engagement.active_ticks`.
     */
    constexpr impl::CounterMetric active_ticks(22);

    /**
     * generated from browser.engagement.uri_count
     */
    /**
     * The number of total non-unique http(s) URIs visited, including page
     * reloads, after the session has been restored. URIs on minimized or
     * background tabs may also be counted. Private browsing uris are included.
     * 
     * Migrated from Telemetry's
     * `browser.engagement.total_uri_count_normal_and_private_mode`.
     */
    constexpr impl::CounterMetric uri_count(23);

}
namespace gmp {
    /**
     * generated from gmp.update_xml_fetch_result
     */
    /**
     * The result of Gecko fetching an update.xml from Balrog. This captures 3
     * different data points: success or failure of the request, if cert pinning or
     * content signatures were used to verify the result, and the reason for failure,
     * if the request failed.
     */
    constexpr impl::Labeled<impl::CounterMetric> update_xml_fetch_result(24);

}
namespace perf {
    /**
     * generated from perf.page_load
     */
    struct PageLoadExtra {
      mozilla::Maybe<uint32_t> fcpTime;
      mozilla::Maybe<uint32_t> jsExecTime;
      mozilla::Maybe<uint32_t> loadTime;
      mozilla::Maybe<nsCString> loadType;
      mozilla::Maybe<uint32_t> responseTime;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (fcpTime) {
          extraKeys.AppendElement()->AssignASCII("fcp_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", fcpTime.value()));
        }
        if (jsExecTime) {
          extraKeys.AppendElement()->AssignASCII("js_exec_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", jsExecTime.value()));
        }
        if (loadTime) {
          extraKeys.AppendElement()->AssignASCII("load_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", loadTime.value()));
        }
        if (loadType) {
          extraKeys.AppendElement()->AssignASCII("load_type");
          extraValues.EmplaceBack(loadType.value());
        }
        if (responseTime) {
          extraKeys.AppendElement()->AssignASCII("response_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", responseTime.value()));
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a top level content document has been loaded.
     */
    constexpr impl::EventMetric<PageLoadExtra> page_load(25);

}
namespace paint {
    /**
     * generated from paint.build_displaylist_time
     */
    /**
     * The time to build a Gecko display list.
     */
    constexpr impl::TimingDistributionMetric build_displaylist_time(26);

}
namespace wr {
    /**
     * generated from wr.gpu_wait_time
     */
    /**
     * The time spent waiting for the GPU to complete previously issued drawing
     * commands.
     */
    constexpr impl::TimingDistributionMetric gpu_wait_time(27);

    /**
     * generated from wr.rasterize_glyphs_time
     */
    /**
     * The time to rasterize glyphs for consumption by WebRender.
     */
    constexpr impl::TimingDistributionMetric rasterize_glyphs_time(28);

    /**
     * generated from wr.rasterize_blobs_time
     */
    /**
     * The time to rasterize blobs for consumption by WebRender.
     */
    constexpr impl::TimingDistributionMetric rasterize_blobs_time(29);

    /**
     * generated from wr.renderer_time
     */
    /**
     * WebRender renderer time.
     */
    constexpr impl::TimingDistributionMetric renderer_time(30);

    /**
     * generated from wr.renderer_time_no_sc
     */
    /**
     * WebRender renderer time excluding frames in which shader compilation took
     * place.
     */
    constexpr impl::TimingDistributionMetric renderer_time_no_sc(31);

    /**
     * generated from wr.framebuild_time
     */
    /**
     * The time to rasterize glyphs for consumption by WebRender.
     */
    constexpr impl::TimingDistributionMetric framebuild_time(32);

    /**
     * generated from wr.scenebuild_time
     */
    /**
     * WebRender scene build time.
     */
    constexpr impl::TimingDistributionMetric scenebuild_time(33);

    /**
     * generated from wr.sceneswap_time
     */
    /**
     * WebRender scene swap time.
     */
    constexpr impl::TimingDistributionMetric sceneswap_time(34);

    /**
     * generated from wr.texture_cache_update_time
     */
    /**
     * Time taken by WebRender to update the texture cache.
     */
    constexpr impl::TimingDistributionMetric texture_cache_update_time(35);

    /**
     * generated from wr.time_to_frame_build
     */
    /**
     * Time elapsed between the construction of a transaction and the associated frame
     * build beginning.
     */
    constexpr impl::TimingDistributionMetric time_to_frame_build(36);

    /**
     * generated from wr.time_to_render_start
     */
    /**
     * Time elapsed between the construction of a frame and the start of rendering.
     */
    constexpr impl::TimingDistributionMetric time_to_render_start(37);

}
namespace netwerk {
    /**
     * generated from netwerk.early_hints
     */
    /**
     * Counts the different type of resources that are sent for early hints.
     */
    constexpr impl::Labeled<impl::CounterMetric> early_hints(38);

}
namespace extensions {
    /**
     * generated from extensions.startup_cache_load_time
     */
    /**
     * Time to load and deserialize the extensions startupCache data.
     */
    constexpr impl::TimespanMetric startup_cache_load_time(39);

}
namespace fog {
    /**
     * generated from fog.initialization
     */
    /**
     * Time the FOG initialization takes.
     */
    constexpr impl::TimespanMetric initialization(40);

    /**
     * generated from fog.failed_idle_registration
     */
    /**
     * True if we failed to register with the idle service. Absent otherwise.
     * Means IPC probably isn't working well.
     * Child-process data will likely be absent, or incomplete.
     */
    constexpr impl::BooleanMetric failed_idle_registration(41);

}
namespace fog_ipc {
    /**
     * generated from fog.ipc.replay_failures
     */
    /**
     * The number of times the ipc buffer failed to be replayed in the
     * parent process.
     */
    constexpr impl::CounterMetric replay_failures(42);

    /**
     * generated from fog.ipc.buffer_sizes
     */
    /**
     * The number and size of the IPC buffers being received over FOG IPC.
     */
    constexpr impl::MemoryDistributionMetric buffer_sizes(43);

    /**
     * generated from fog.ipc.flush_durations
     */
    /**
     * The length of time between asking the child processes for their
     * IPC buffers and all of them being received by the parent.
     */
    constexpr impl::TimingDistributionMetric flush_durations(44);

    /**
     * generated from fog.ipc.flush_failures
     */
    /**
     * The number of times we failed to flush all non-parent-process data,
     * throwing even partial results into the trash.
     * If this number is high, we might consider writing custom `MozPromise`-
     * handling code instead of using `MozPromise::All`.
     */
    constexpr impl::CounterMetric flush_failures(45);

    /**
     * generated from fog.ipc.shutdown_registration_failures
     */
    /**
     * The number of times we tried to register shutdown flush routines for
     * content child processes, and failed (probably because there was no main
     * thread).
     * As a result there may be data loss from content child processes.
     * Large or rising number of clients experiencing this indicates we should
     * perhaps refactor content child shutdown in FOG to try harder to register
     * flush operations.
     * Will likely be obsoleted by bug 1641989.
     */
    constexpr impl::CounterMetric shutdown_registration_failures(46);

}
namespace test_only {
    /**
     * generated from test_only.bad_code
     */
    /**
     * Number of times we encountered bad code.
     * This is a test-only metric.
     */
    constexpr impl::CounterMetric bad_code(47);

    /**
     * generated from test_only.can_we_time_it
     */
    /**
     * Test metric for a timespan.
     * This is a test-only metric.
     */
    constexpr impl::TimespanMetric can_we_time_it(48);

    /**
     * generated from test_only.cheesy_string
     */
    /**
     * Only the cheesiest of strings.
     * This is a test-only metric.
     */
    constexpr impl::StringMetric cheesy_string(49);

    /**
     * generated from test_only.cheesy_string_list
     */
    /**
     * Only the cheesiest of strings. In list form!
     * This is a test-only metric.
     */
    constexpr impl::StringListMetric cheesy_string_list(50);

    /**
     * generated from test_only.what_a_date
     */
    /**
     * ...To be writing FOG code. This is a test-only metric.
     */
    constexpr impl::DatetimeMetric what_a_date(51);

    /**
     * generated from test_only.what_id_it
     */
    /**
     * Just a UUID.
     * This is a test-only metric.
     */
    constexpr impl::UuidMetric what_id_it(52);

    /**
     * generated from test_only.can_we_flag_it
     */
    /**
     * Test metric for a boolean.
     * This is a test-only metric.
     */
    constexpr impl::BooleanMetric can_we_flag_it(53);

    /**
     * generated from test_only.do_you_remember
     */
    /**
     * They say it's the second thing to go.
     * This is a test-only metric.
     */
    constexpr impl::MemoryDistributionMetric do_you_remember(54);

    /**
     * generated from test_only.what_time_is_it
     */
    /**
     * Adheres to at least two of the top ten fallacies programmers believe
     * about time.
     * This is a test-only metric.
     */
    constexpr impl::TimingDistributionMetric what_time_is_it(55);

    /**
     * generated from test_only.mabels_kitchen_counters
     */
    /**
     * Counts Mabels labeled by their kitchen counters.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::CounterMetric> mabels_kitchen_counters(56);

    /**
     * generated from test_only.mabels_bathroom_counters
     */
    /**
     * Counts Mabels labeled by their bathroom counters.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::CounterMetric> mabels_bathroom_counters(57);

    /**
     * generated from test_only.mabels_like_balloons
     */
    /**
     * Does the labeled Mabel like balloons?
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::BooleanMetric> mabels_like_balloons(58);

    /**
     * generated from test_only.mabels_balloon_strings
     */
    /**
     * What do the labeled Mabel's liked balloons' strings say?
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::StringMetric> mabels_balloon_strings(59);

    /**
     * generated from test_only.mabels_label_maker
     */
    /**
     * Mabel just got a label maker and wants to party like it's
     * 1999.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::StringMetric> mabels_label_maker(60);

    /**
     * generated from test_only.mirror_time
     */
    /**
     * Mirrored metric for a timespan.
     * This is a test-only metric.
     */
    constexpr impl::TimespanMetric mirror_time(61);

    /**
     * generated from test_only.mirror_time_nanos
     */
    /**
     * Mirrored metric for a timespan.
     * This is a test-only metric.
     */
    constexpr impl::TimespanMetric mirror_time_nanos(62);

    /**
     * generated from test_only.mirrors_for_labeled_bools
     */
    /**
     * Mirrored metric.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::BooleanMetric> mirrors_for_labeled_bools(63);

    /**
     * generated from test_only.one_ping_one_bool
     */
    /**
     * One bool for one ping only.
     * This is a test-only metric.
     */
    constexpr impl::BooleanMetric one_ping_one_bool(64);

    /**
     * generated from test_only.meaning_of_life
     */
    /**
     * Measures the one true answer to the Ultimate Question of Life,
     * the Universe, and Everything.
     * Approximately.
     * This is a test-only metric.
     */
    constexpr impl::QuantityMetric meaning_of_life(65);

}
namespace test_only_ipc {
    /**
     * generated from test_only.ipc.a_counter
     */
    /**
     * This is a test-only metric.
     * Just counting things.
     */
    constexpr impl::CounterMetric a_counter(66);

    /**
     * generated from test_only.ipc.a_bool
     */
    /**
     * This is a test-only metric.
     * Just flagging things.
     */
    constexpr impl::BooleanMetric a_bool(67);

    /**
     * generated from test_only.ipc.a_date
     */
    /**
     * This is a test-only metric.
     * Just putting things on the calendar.
     */
    constexpr impl::DatetimeMetric a_date(68);

    /**
     * generated from test_only.ipc.a_string
     */
    /**
     * This is a test-only metric.
     * Just setting some strings.
     */
    constexpr impl::StringMetric a_string(69);

    /**
     * generated from test_only.ipc.a_memory_dist
     */
    /**
     * This is a test-only metric.
     * Just measuring memory.
     */
    constexpr impl::MemoryDistributionMetric a_memory_dist(70);

    /**
     * generated from test_only.ipc.a_timing_dist
     */
    /**
     * This is a test-only metric.
     * Just measuring time.
     */
    constexpr impl::TimingDistributionMetric a_timing_dist(71);

    /**
     * generated from test_only.ipc.a_custom_dist
     */
    /**
     * This is a test-only metric.
     * Just measuring samples.
     */
    constexpr impl::CustomDistributionMetric a_custom_dist(72);

    /**
     * generated from test_only.ipc.a_string_list
     */
    /**
     * This is a test-only metric.
     * Just appending some strings.
     */
    constexpr impl::StringListMetric a_string_list(73);

    /**
     * generated from test_only.ipc.an_event
     */
    struct AnEventExtra {
      mozilla::Maybe<nsCString> extra1;
      mozilla::Maybe<nsCString> extra2;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (extra1) {
          extraKeys.AppendElement()->AssignASCII("extra1");
          extraValues.EmplaceBack(extra1.value());
        }
        if (extra2) {
          extraKeys.AppendElement()->AssignASCII("extra2");
          extraValues.EmplaceBack(extra2.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * This is a test-only metric.
     * Just recording some events.
     */
    constexpr impl::EventMetric<AnEventExtra> an_event(74);

    /**
     * generated from test_only.ipc.event_with_extra
     */
    struct EventWithExtraExtra {
      mozilla::Maybe<nsCString> extra1;
      mozilla::Maybe<uint32_t> extra2;
      mozilla::Maybe<bool> extra3LongerName;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (extra1) {
          extraKeys.AppendElement()->AssignASCII("extra1");
          extraValues.EmplaceBack(extra1.value());
        }
        if (extra2) {
          extraKeys.AppendElement()->AssignASCII("extra2");
          extraValues.EmplaceBack(nsPrintfCString("%d", extra2.value()));
        }
        if (extra3LongerName) {
          extraKeys.AppendElement()->AssignASCII("extra3_longer_name");
          extraValues.AppendElement()->AssignASCII(extra3LongerName.value() ? "true" : "false");
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * This is a test-only metric.
     * Just recording some events.
     */
    constexpr impl::EventMetric<EventWithExtraExtra> event_with_extra(75);

    /**
     * generated from test_only.ipc.no_extra_event
     */
    /**
     * This is a test-only metric.
     * Just recording some events without the extra fuss.
     */
    constexpr impl::EventMetric<NoExtraKeys> no_extra_event(76);

    /**
     * generated from test_only.ipc.a_uuid
     */
    /**
     * This is a test-only metric.
     * Just recording some unique identifiers.
     */
    constexpr impl::UuidMetric a_uuid(77);

    /**
     * generated from test_only.ipc.a_labeled_counter
     */
    /**
     * This is a test-only metric.
     * Just counting labeled things.
     */
    constexpr impl::Labeled<impl::CounterMetric> a_labeled_counter(78);

    /**
     * generated from test_only.ipc.another_labeled_counter
     */
    /**
     * This is a test-only metric.
     * Just another metric counting labeled things.
     */
    constexpr impl::Labeled<impl::CounterMetric> another_labeled_counter(79);

    /**
     * generated from test_only.ipc.a_quantity
     */
    /**
     * This is a test-only metric.
     * Just quantifying things.
     */
    constexpr impl::QuantityMetric a_quantity(80);

    /**
     * generated from test_only.ipc.irate
     */
    /**
     * This is a test-only metric.
     * A rate that isn't happy about it.
     */
    constexpr impl::RateMetric irate(81);

    /**
     * generated from test_only.ipc.rate_with_external_denominator
     */
    /**
     * This is a test-only metric.
     * A rate with a denominator that is Out There.
     */
    constexpr impl::NumeratorMetric rate_with_external_denominator(82);

    /**
     * generated from test_only.ipc.an_external_denominator
     */
    /**
     * This is a test-only metric.
     * A denominator not from around here.
     */
    constexpr impl::DenominatorMetric an_external_denominator(83);

    /**
     * generated from test_only.ipc.a_url
     */
    /**
     * This is a test-only metric.
     * Just setting some Urls.
     */
    constexpr impl::UrlMetric a_url(84);

}
namespace nimbus_events {
    /**
     * generated from nimbus_events.enrollment
     */
    struct EnrollmentExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> enrollmentId;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> experimentType;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (enrollmentId) {
          extraKeys.AppendElement()->AssignASCII("enrollment_id");
          extraValues.EmplaceBack(enrollmentId.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (experimentType) {
          extraKeys.AppendElement()->AssignASCII("experiment_type");
          extraValues.EmplaceBack(experimentType.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a user has met the conditions and is first bucketed into an
     * experiment (i.e. targeting matched and they were randomized into a bucket and
     * branch of the experiment). Expected a maximum of once per experiment per user.
     */
    constexpr impl::EventMetric<EnrollmentExtra> enrollment(85);

    /**
     * generated from nimbus_events.enroll_failed
     */
    struct EnrollFailedExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when an enrollment fails, including the reason for the failure.
     */
    constexpr impl::EventMetric<EnrollFailedExtra> enroll_failed(86);

    /**
     * generated from nimbus_events.unenrollment
     */
    struct UnenrollmentExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> enrollmentId;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (enrollmentId) {
          extraKeys.AppendElement()->AssignASCII("enrollment_id");
          extraValues.EmplaceBack(enrollmentId.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when either telemetry is disabled, or the experiment has run for its
     * designed duration (i.e. it is no longer present in the Nimbus Remote Settings
     * collection)
     */
    constexpr impl::EventMetric<UnenrollmentExtra> unenrollment(87);

    /**
     * generated from nimbus_events.unenroll_failed
     */
    struct UnenrollFailedExtra {
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when an unenrollment fails, including the reason for the failure.
     */
    constexpr impl::EventMetric<UnenrollFailedExtra> unenroll_failed(88);

    /**
     * generated from nimbus_events.exposure
     */
    struct ExposureExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> featureId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (featureId) {
          extraKeys.AppendElement()->AssignASCII("feature_id");
          extraValues.EmplaceBack(featureId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a user actually observes an experimental treatment, or would have
     * observed an experimental treatment if they had been in a branch that would have
     * shown one.
     */
    constexpr impl::EventMetric<ExposureExtra> exposure(89);

    /**
     * generated from nimbus_events.validation_failed
     */
    struct ValidationFailedExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> feature;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (feature) {
          extraKeys.AppendElement()->AssignASCII("feature");
          extraValues.EmplaceBack(feature.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * This records when validation of a recipe fails.
     */
    constexpr impl::EventMetric<ValidationFailedExtra> validation_failed(90);

}
namespace pdfjs {
    /**
     * generated from pdfjs.editing
     */
    /**
     * Counts the number of times some PDF editing features are used.
     */
    constexpr impl::Labeled<impl::CounterMetric> editing(91);

    /**
     * generated from pdfjs.buttons
     */
    /**
     * Counts the number of times some actions are executed in the PDF viewer.
     */
    constexpr impl::Labeled<impl::CounterMetric> buttons(92);

}
namespace power {
    /**
     * generated from power.cpu_time_bogus_values
     */
    /**
     * Impossibly large CPU time values that were discarded.
     */
    constexpr impl::CounterMetric cpu_time_bogus_values(93);

    /**
     * generated from power.cpu_time_per_process_type_ms
     */
    /**
     * CPU time used by each process type in ms.
     */
    constexpr impl::Labeled<impl::CounterMetric> cpu_time_per_process_type_ms(94);

    /**
     * generated from power.gpu_time_per_process_type_ms
     */
    /**
     * GPU time used by each process type in ms.
     */
    constexpr impl::Labeled<impl::CounterMetric> gpu_time_per_process_type_ms(95);

    /**
     * generated from power.gpu_time_bogus_values
     */
    /**
     * Impossibly large GPU time values that were discarded.
     */
    constexpr impl::CounterMetric gpu_time_bogus_values(96);

    /**
     * generated from power.wakeups_per_process_type
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> wakeups_per_process_type(97);

    /**
     * generated from power.total_cpu_time_ms
     */
    /**
     * Total CPU time used by all processes in ms.
     */
    constexpr impl::CounterMetric total_cpu_time_ms(98);

    /**
     * generated from power.total_gpu_time_ms
     */
    /**
     * Total GPU time used by all processes in ms.
     */
    constexpr impl::CounterMetric total_gpu_time_ms(99);

    /**
     * generated from power.total_thread_wakeups
     */
    /**
     * How many times threads woke up and could have woken up a CPU core.
     */
    constexpr impl::CounterMetric total_thread_wakeups(100);

}
namespace power_wakeups_per_thread {
    /**
     * generated from power.wakeups_per_thread.parent_active
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_active(101);

    /**
     * generated from power.wakeups_per_thread.parent_inactive
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_inactive(102);

    /**
     * generated from power.wakeups_per_thread.content_foreground
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_foreground(103);

    /**
     * generated from power.wakeups_per_thread.content_background
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_background(104);

    /**
     * generated from power.wakeups_per_thread.gpu_process
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> gpu_process(105);

}
namespace power_cpu_ms_per_thread {
    /**
     * generated from power.cpu_ms_per_thread.parent_active
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_active(106);

    /**
     * generated from power.cpu_ms_per_thread.parent_inactive
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_inactive(107);

    /**
     * generated from power.cpu_ms_per_thread.content_foreground
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_foreground(108);

    /**
     * generated from power.cpu_ms_per_thread.content_background
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_background(109);

    /**
     * generated from power.cpu_ms_per_thread.gpu_process
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> gpu_process(110);

}
namespace power_battery {
    /**
     * generated from power.battery.percentage_when_user_active
     */
    /**
     * Records how many percent of battery was available for each period of user
     * activity.
     */
    constexpr impl::CustomDistributionMetric percentage_when_user_active(111);

}
namespace search_engine_default {
    /**
     * generated from search.engine.default.engine_id
     */
    /**
     * The telemetry id of the search engine.
     * For application provided engines, this is either supplied by the
     * configuration or from the first part of the associated WebExtension Id.
     * For other engines, this is `other-<extensionName>`.
     */
    constexpr impl::StringMetric engine_id(112);

    /**
     * generated from search.engine.default.display_name
     */
    /**
     * The display name of the user's default engine.
     */
    constexpr impl::StringMetric display_name(113);

    /**
     * generated from search.engine.default.load_path
     */
    /**
     * A path relating to where the search engine was installed/loaded from.
     * For example:
     *   `[other]addEngineWithDetails:<extension id>` for a WebExtension based
     *   engine.
     *   `[https]developer.mozilla.org/mdn-web-docs.xml` for an OpenSearch based
     *   engine.
     * Note: this metric is truncated at 100 characters.
     */
    constexpr impl::StringMetric load_path(114);

    /**
     * generated from search.engine.default.submission_url
     */
    /**
     * The submission URL of the default engine. This is only reported in the
     * cases where:
     *   The engine is an application provided engine.
     *   The engine has the same name as an application provided engine.
     *   The engine matches one of a specific list of well known search engines.
     */
    constexpr impl::UrlMetric submission_url(115);

    /**
     * generated from search.engine.default.verified
     */
    /**
     * The verified status of the search engine.
     * For application provided engines, this will always be `default`.
     * For other engines this will either be `verified` or `unverified` depending
     * on if the loadPathHash is valid.
     */
    constexpr impl::StringMetric verified(116);

    /**
     * generated from search.engine.default.changed
     */
    struct ChangedExtra {
      mozilla::Maybe<nsCString> changeSource;
      mozilla::Maybe<nsCString> newDisplayName;
      mozilla::Maybe<nsCString> newEngineId;
      mozilla::Maybe<nsCString> newLoadPath;
      mozilla::Maybe<nsCString> newSubmissionUrl;
      mozilla::Maybe<nsCString> previousEngineId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (changeSource) {
          extraKeys.AppendElement()->AssignASCII("change_source");
          extraValues.EmplaceBack(changeSource.value());
        }
        if (newDisplayName) {
          extraKeys.AppendElement()->AssignASCII("new_display_name");
          extraValues.EmplaceBack(newDisplayName.value());
        }
        if (newEngineId) {
          extraKeys.AppendElement()->AssignASCII("new_engine_id");
          extraValues.EmplaceBack(newEngineId.value());
        }
        if (newLoadPath) {
          extraKeys.AppendElement()->AssignASCII("new_load_path");
          extraValues.EmplaceBack(newLoadPath.value());
        }
        if (newSubmissionUrl) {
          extraKeys.AppendElement()->AssignASCII("new_submission_url");
          extraValues.EmplaceBack(newSubmissionUrl.value());
        }
        if (previousEngineId) {
          extraKeys.AppendElement()->AssignASCII("previous_engine_id");
          extraValues.EmplaceBack(previousEngineId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when the default search engine is changed.
     */
    constexpr impl::EventMetric<ChangedExtra> changed(117);

}
namespace search_engine_private {
    /**
     * generated from search.engine.private.engine_id
     */
    /**
     * The telemetry id of the search engine.
     * For application provided engines, this is either supplied by the
     * configuration or from the first part of the associated WebExtension Id.
     * For other engines, this is `other-<extensionName>`.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric engine_id(118);

    /**
     * generated from search.engine.private.display_name
     */
    /**
     * The display name of the user's default engine.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric display_name(119);

    /**
     * generated from search.engine.private.load_path
     */
    /**
     * A path relating to where the search engine was installed/loaded from.
     * For example:
     *   `[other]addEngineWithDetails:<extension id>` for a WebExtension based
     *   engine.
     *   `[https]developer.mozilla.org/mdn-web-docs.xml` for an OpenSearch based
     *   engine.
     * Note: this metric is truncated at 100 characters.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric load_path(120);

    /**
     * generated from search.engine.private.submission_url
     */
    /**
     * The submission URL of the default engine. This is only reported in the
     * cases where:
     *   The engine is an application provided engine.
     *   The engine has the same name as an application provided engine.
     *   The engine matches one of a specific list of well known search engines.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::UrlMetric submission_url(121);

    /**
     * generated from search.engine.private.verified
     */
    /**
     * The verified status of the search engine.
     * For application provided engines, this will always be `default`.
     * For other engines this will either be `verified` or `unverified` depending
     * on if the loadPathHash is valid.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric verified(122);

    /**
     * generated from search.engine.private.changed
     */
    struct ChangedExtra {
      mozilla::Maybe<nsCString> changeSource;
      mozilla::Maybe<nsCString> newDisplayName;
      mozilla::Maybe<nsCString> newEngineId;
      mozilla::Maybe<nsCString> newLoadPath;
      mozilla::Maybe<nsCString> newSubmissionUrl;
      mozilla::Maybe<nsCString> previousEngineId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (changeSource) {
          extraKeys.AppendElement()->AssignASCII("change_source");
          extraValues.EmplaceBack(changeSource.value());
        }
        if (newDisplayName) {
          extraKeys.AppendElement()->AssignASCII("new_display_name");
          extraValues.EmplaceBack(newDisplayName.value());
        }
        if (newEngineId) {
          extraKeys.AppendElement()->AssignASCII("new_engine_id");
          extraValues.EmplaceBack(newEngineId.value());
        }
        if (newLoadPath) {
          extraKeys.AppendElement()->AssignASCII("new_load_path");
          extraValues.EmplaceBack(newLoadPath.value());
        }
        if (newSubmissionUrl) {
          extraKeys.AppendElement()->AssignASCII("new_submission_url");
          extraValues.EmplaceBack(newSubmissionUrl.value());
        }
        if (previousEngineId) {
          extraKeys.AppendElement()->AssignASCII("previous_engine_id");
          extraValues.EmplaceBack(previousEngineId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when the default search engine is changed
     */
    constexpr impl::EventMetric<ChangedExtra> changed(123);

}
namespace legacy_telemetry {
    /**
     * generated from legacy.telemetry.client_id
     */
    /**
     * The client_id according to Telemetry.
     * Might not always have a value due to being too early for it to have
     * loaded.
     * Value may be the canary client id `c0ffeec0-ffee-c0ff-eec0-ffeec0ffeec0`
     * in pings near when the data upload pref is disabled (if Telemetry gets
     * to go first), or between when a client_id has been removed and when it
     * has been regenerated.
     * Does not need to be sent in the Glean "deletion-request" ping.
     */
    constexpr impl::UuidMetric client_id(124);

    /**
     * generated from legacy.telemetry.profile_subsession_counter
     */
    /**
     * The `profileSubsessionCounter` from the Legacy Telemetry "main" ping.
     * This is a value that monotonically increases once for every "main" ping
     * that has been submitted.
     * It is a sequence number by a longer name.
     */
    constexpr impl::QuantityMetric profile_subsession_counter(125);

}
namespace background_update {
    /**
     * generated from background_update.client_id
     */
    /**
     * The Telemetry client ID of the default profile.
     */
    constexpr impl::UuidMetric client_id(126);

    /**
     * generated from background_update.final_state
     */
    /**
     * String description of the final state the update state machine reached.
     */
    constexpr impl::StringMetric final_state(127);

    /**
     * generated from background_update.states
     */
    /**
     * Ordered list of string descriptions of the states that the update state machine
     * reached.
     */
    constexpr impl::StringListMetric states(128);

    /**
     * generated from background_update.reasons
     */
    /**
     * List of reasons that the background update task did not run.
     */
    constexpr impl::StringListMetric reasons(129);

    /**
     * generated from background_update.exit_code_success
     */
    /**
     * True if the exit code/status of the background update task is 0, which means
     * success.
     */
    constexpr impl::BooleanMetric exit_code_success(130);

    /**
     * generated from background_update.exit_code_exception
     */
    /**
     * True if the exit code/status of the background update task is 3, which means an
     * exception was thrown.
     */
    constexpr impl::BooleanMetric exit_code_exception(131);

}
namespace update {
    /**
     * generated from update.service_enabled
     */
    /**
     * Preference "app.update.service.enabled": whether the Mozilla Maintenance
     * Service is enabled.
     */
    constexpr impl::BooleanMetric service_enabled(132);

    /**
     * generated from update.auto_download
     */
    /**
     * Per-installation preference "app.update.auto": whether to fetch and install
     * updates without user intervention.
     */
    constexpr impl::BooleanMetric auto_download(133);

    /**
     * generated from update.background_update
     */
    /**
     * Per-installation preference "app.update.background.enabled": whether to fetch
     * and install updates in the background when Firefox is not running.
     */
    constexpr impl::BooleanMetric background_update(134);

    /**
     * generated from update.enabled
     */
    /**
     * True when policies are disabled or when the "DisableAppUpdate" is not in
     * effect.
     */
    constexpr impl::BooleanMetric enabled(135);

    /**
     * generated from update.channel
     */
    /**
     * The update channel.
     */
    constexpr impl::StringMetric channel(136);

    /**
     * generated from update.can_usually_apply_updates
     */
    /**
     * Whether or not the Update Service can usually download and install updates. See
     * `canUsuallyApplyUpdates` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_apply_updates(137);

    /**
     * generated from update.can_usually_check_for_updates
     */
    /**
     * Whether or not the Update Service can usually check for updates. See
     * `canUsuallyCheckForUpdates` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_check_for_updates(138);

    /**
     * generated from update.can_usually_stage_updates
     */
    /**
     * Whether the Update Service is usually able to stage updates. See
     * `canUsuallyStageUpdates` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_stage_updates(139);

    /**
     * generated from update.can_usually_use_bits
     */
    /**
     * On Windows, whether the Update Service can usually use BITS. See
     * `canUsuallyUseBits` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_use_bits(140);

}
namespace startup {
    /**
     * generated from startup.run_from_dmg_install_outcome
     */
    /**
     * The outcome after the app detected that it was running from DMG or an app
     * translocated location and should offer to install and relaunch itself.
     */
    constexpr impl::Labeled<impl::BooleanMetric> run_from_dmg_install_outcome(141);

}

} // namespace mozilla::glean

#endif // mozilla_Metrics_h
